@@ -9,7 +9,14 @@ from collections import defaultdict
 
 import django.db.transaction
 from django.core.management import BaseCommand
-from stellar_sdk import Keypair, TransactionEnvelope, Asset, Claimant, Server
+from stellar_sdk import (
+    Keypair,
+    TransactionEnvelope,
+    Asset,
+    Claimant,
+    Server,
+    MuxedAccount,
+)
 from stellar_sdk.client.aiohttp_client import AiohttpClient
 from stellar_sdk.account import Account
 from stellar_sdk.exceptions import (
@@ -255,7 +262,10 @@ class PendingDeposits:
                     Transaction.STATUS.pending_user_transfer_start,
                     Transaction.STATUS.pending_external,
                 ],
-                kind=Transaction.KIND.deposit,
+                kind__in=[
+                    Transaction.KIND.deposit,
+                    getattr(Transaction.KIND, "deposit-exchange"),
+                ],
                 pending_execution_attempt=False,
             )
             .select_related("asset")
@@ -268,12 +278,31 @@ class PendingDeposits:
             ).update(pending_execution_attempt=True)
         verified_ready_transactions = []
         for transaction in ready_transactions:
+            if not transaction.amount_fee or not transaction.amount_out:
+                if transaction.quote:
+                    logger.error(
+                        f"transaction {transaction.id} uses a quote but was returned "
+                        "from poll_pending_deposits() without amount_fee or amount_out "
+                        "assigned, skipping"
+                    )
+                    continue
+                logger.warning(
+                    f"transaction {transaction.id} was returned from "
+                    f"poll_pending_deposits() without Transaction.amount_fee or "
+                    f"Transaction.amount_out assigned. Future Polaris "
+                    "releases will not calculate fees and delivered amounts"
+                )
             # refresh from DB to pull pending_execution_attempt value and to ensure invalid
             # values were not assigned to the transaction in rri.poll_pending_deposits()
             asset = transaction.asset
+            quote = transaction.quote
             transaction.refresh_from_db()
             transaction.asset = asset
-            if transaction.kind != transaction.KIND.deposit:
+            transaction.quote = quote
+            if transaction.kind not in [
+                transaction.KIND.deposit,
+                getattr(transaction.KIND, "deposit-exchange"),
+            ]:
                 cls.handle_error(
                     transaction,
                     "poll_pending_deposits() returned a non-deposit transaction",
@@ -298,10 +327,8 @@ class PendingDeposits:
                                 "asset_code": transaction.asset.code,
                             }
                         )
-                    except ValueError as e:
-                        cls.handle_error(transaction, str(e))
-                        maybe_make_callback(transaction)
-                        continue
+                    except ValueError:
+                        transaction.amount_fee = Decimal(0)
                 else:
                     transaction.amount_fee = Decimal(0)
                 transaction.save()
@@ -402,9 +429,15 @@ class PendingDeposits:
             logger.debug(
                 f"got lock to get or create destination account for transaction {transaction.id}"
             )
+            if transaction.to_address.startswith("M"):
+                destination_account = MuxedAccount.from_account(
+                    transaction.to_address
+                ).account_id
+            else:
+                destination_account = transaction.to_address
             try:
                 account, json_resp = await get_account_obj_async(
-                    Keypair.from_public_key(transaction.to_address), server
+                    Keypair.from_public_key(destination_account), server
                 )
                 logger.debug(f"account for transaction {transaction.id} exists")
                 return account, is_pending_trust(transaction, json_resp)
@@ -486,12 +519,18 @@ class PendingDeposits:
         transaction is scheduled for processing. If not, the transaction is
         updated to no longer be pending an execution attempt.
         """
+        if transaction.to_address.startswith("M"):
+            destination_account = MuxedAccount.from_account(
+                transaction.to_address
+            ).account_id
+        else:
+            destination_account = transaction.to_address
         try:
             _, account = await get_account_obj_async(
-                Keypair.from_public_key(transaction.to_address), server
+                Keypair.from_public_key(destination_account), server
             )
         except BaseRequestError:
-            logger.exception(f"Failed to load account {transaction.to_address}")
+            logger.exception(f"Failed to load account {destination_account}")
             transaction.pending_execution_attempt = False
             await sync_to_async(transaction.save)()
             return
@@ -595,11 +634,12 @@ class PendingDeposits:
         transaction.stellar_transaction_id = response["id"]
         transaction.status = Transaction.STATUS.completed
         transaction.completed_at = datetime.datetime.now(datetime.timezone.utc)
-        transaction.amount_out = round(
-            Decimal(transaction.amount_in) - Decimal(transaction.amount_fee),
-            transaction.asset.significant_decimals,
-        )
         transaction.pending_execution_attempt = False
+        if not transaction.quote:
+            transaction.amount_out = round(
+                Decimal(transaction.amount_in) - Decimal(transaction.amount_fee),
+                transaction.asset.significant_decimals,
+            )
         await sync_to_async(transaction.save)()
         logger.info(f"transaction {transaction.id} completed.")
         await maybe_make_callback_async(transaction)
@@ -629,10 +669,18 @@ class PendingDeposits:
     async def create_deposit_envelope(
         transaction, source_account, server
     ) -> TransactionEnvelope:
-        payment_amount = round(
-            Decimal(transaction.amount_in) - Decimal(transaction.amount_fee),
-            transaction.asset.significant_decimals,
-        )
+        if transaction.amount_out:
+            payment_amount = transaction.amount_out
+        elif transaction.quote:
+            raise RuntimeError(
+                f"transaction {transaction.id} uses a quote but does not have "
+                "amount_out assigned"
+            )
+        else:
+            payment_amount = round(
+                Decimal(transaction.amount_in) - Decimal(transaction.amount_fee),
+                transaction.asset.significant_decimals,
+            )
         builder = TransactionBuilder(
             source_account=source_account,
             network_passphrase=settings.STELLAR_NETWORK_PASSPHRASE,
@@ -769,8 +817,10 @@ class PendingDeposits:
             rdi.create_channel_account(transaction=transaction)
         if not transaction.channel_seed:
             asset = transaction.asset
+            quote = transaction.quote
             transaction.refresh_from_db()
             transaction.asset = asset
+            transaction.quote = quote
         return Keypair.from_secret(transaction.channel_seed)
 
     @classmethod
