@@ -21,17 +21,14 @@ def _get_asset_info(asset: Asset, op_type: str) -> Dict:
     max_amount_attr = f"{op_type}_max_amount"
     min_amount = getattr(asset, min_amount_attr)
     max_amount = getattr(asset, max_amount_attr)
-    if min_amount > Asset._meta.get_field(min_amount_attr).default:
+    if min_amount > getattr(Asset, "_meta").get_field(min_amount_attr).default:
         asset_info["min_amount"] = min_amount
-    if max_amount < Asset._meta.get_field(max_amount_attr).default:
+    if max_amount < getattr(Asset, "_meta").get_field(max_amount_attr).default:
         asset_info["max_amount"] = max_amount
-    if registered_fee_func is calculate_fee:
-        # the anchor has not replaced the default fee function
-        # so `fee_fixed` and `fee_percent` are still relevant.
-        asset_info.update(
-            fee_fixed=getattr(asset, f"{op_type}_fee_fixed"),
-            fee_percent=getattr(asset, f"{op_type}_fee_percent"),
-        )
+    if getattr(asset, f"{op_type}_fee_fixed") is not None:
+        asset_info["fee_fixed"] = getattr(asset, f"{op_type}_fee_fixed")
+    if getattr(asset, f"{op_type}_fee_percent") is not None:
+        asset_info["fee_percent"] = getattr(asset, f"{op_type}_fee_percent")
 
     return asset_info
 
